local M = {}

local timer = 0
local running = false
local failed = false
local maxTime = 7

local function showFailedbyTime()
	guihooks.trigger('ScenarioFlashMessage', {{'Used more than '..tostring(maxTime)..' sec', 2}} )
end

local function onRaceInit( sc )
	running = false	
	failed = false
end

-- called when countdown finished
local function onRaceStart()
	timer = 0
	running = true
end

-- data { vehicleName = 'string', waypointName = 'string' }
local function onRaceWaypoint( status ) 
	--dump(data)
end

-- called every 250ms. Use "status.setScenarioFailed()" for return failed test
-- data { dTime = number }
local function onRaceTick( status )
	
	if not running then 
		return true
	end
	timer = timer + status.dTime

	if not failed and timer > maxTime then
		failed = true
	end
	
    -- let some time for onRaceResult before set as FAILED
	if failed and timer > maxTime + 3 then
		showFailedbyTime()
		running = false
		status.setScenarioFailed()
	end
end


local function onRaceLap( status )
	-- stop running and onRaceTick
	running = false
end

-- called 3 seg after end waypoint. Use "status.setScenarioFailed()" for return failed test
local function onRaceResult( status )

	local failZone1 = scenetree.findObject('failzone_car1')
	local failZone2 = scenetree.findObject('failzone_car2')
	local failZone3 = scenetree.findObject('failzone_curb')
	local vehicle = scenetree.findObject('scenario_player0')
	if not failZone1 or not vehicle then return true end
	if not failZone2 or not vehicle then return true end
	if not failZone3 or not vehicle then return true end
	
	local distance1 =  math.abs( ( failZone1:getPosition() - vehicle:getPosition() ):len() )
	local distance2 =  math.abs( ( failZone2:getPosition() - vehicle:getPosition() ):len() )
	local distance3 =  math.abs( ( failZone3:getPosition() - vehicle:getPosition() ):len() )
	if distance1 < 2 or distance2 < 2 or distance3 < 1 then
		guihooks.trigger('ScenarioFlashMessage', {{'Not parked properly', 2}} )
		status.setScenarioFailed()
	end	

	-- check if we failed by time
	if failed then
		showFailedbyTime()
		status.setScenarioFailed()
	end

end

-- called after check scenario goals
local function onRaceEnd()
	running = false
end

M.onRaceInit = onRaceInit
M.onRaceStart = onRaceStart
M.onRaceEnd = onRaceEnd
M.onRaceWaypoint = onRaceWaypoint
M.onRaceTick = onRaceTick
M.onRaceLap = onRaceLap
M.onRaceResult = onRaceResult

return M
